#property copyright "public domain"

#property indicator_chart_window
//#include <common_functions.mqh>
  
extern color NeutralColor = LightGray;
extern color BullColor = MediumSeaGreen;
extern color ExtremeBullColor = Lime;
extern color BearColor = Orange;
extern color ExtremeBearColor = Red;
  
extern int TmaPeriod = 56;
extern int TmaAtrPeriod = 100;  
 
extern int TmaAtrThreshold = 2;
extern double TmaSlopeThreshold = 0.5; 

extern int LinearRegressionPeriod = 7; 

extern double PivotThreshold = 10; 


extern int Size = 13;
extern string Font = "Arial";  
extern bool ShowExtraInfo = true;

extern bool ShowSlopeChange = true; 
extern bool ShowLinearPriceChange = true; 
extern bool ShowMACrossover = true;
extern bool ShowHA = true;
extern int MaPeriod = 10;  
//This is the same as 4 in the original TMA MACross
extern int MaShift = 1;


int Bottom = 25;
double Tick = 0;
bool AdditionalDigit;

double Pivots[];

double Pivot = 0;
double Resistance1 = 0;
double Resistance2 = 0;
double Resistance3 = 0;
double Resistance4 = 0;
double Resistance5 = 0;
double Support1 = 0;
double Support2 = 0;

int LastPivotDay= 0;
//+------------------------------------------------------------------+
//     expert initialization function                                |       
//+------------------------------------------------------------------+
int init()
  { 
  
   ArrayResize(Pivots,11);
	Tick = MarketInfo(Symbol(), MODE_TICKSIZE);	
   AdditionalDigit = MarketInfo(Symbol(), MODE_MARGINCALCMODE) == 0 && MarketInfo(Symbol(), MODE_PROFITCALCMODE) == 0 && Digits % 2 == 1;
   if (AdditionalDigit) {
        Tick *= 10;
    }    
       
   initGraph();
   return(0);                                                              
  }
  
int deinit()
  {
   deinitGraph();
   Print("shutdown error - ",GetLastError());                               
   return(0);                                                             
  }
int start()
  {
   main();
   return(0);                                                               
  }
  
void main()                                                             
  {   
   RefreshRates();  
   //General Info   
   if (ShowExtraInfo)
   { 
      double spread=NormalizeDouble(((Ask-Bid)/Point)/10,1);
      double atr = iATR(Symbol(),PERIOD_D1, 14,0);
      double price = NormalizeDouble(Close[0],4);     
      atr =  NormalizeDouble((atr/Point)/10,1);
   
      paintGeneral("SpreadValue", spread, NeutralColor, 1);
      paintGeneral("AtrValue", atr, NeutralColor);
      paintPrice(price); 
   }
   
   //Tma Info
   GetPivots(Symbol());
   double tma = getTma(Symbol(),0, 0); 
   double tmaPrev  = getTma(Symbol(),0, 1);
   double tmaPrev2  = getTma(Symbol(),0, 2);
   double tmaPrev3  = getTma(Symbol(),0, 3);
   double priceSlope  = getPriceSlope(Symbol(),0,LinearRegressionPeriod);
   double pivotDist = getNearestPivotDistance()/Tick;
   
   double tmaAtr = iATR( Symbol(), 0, TmaAtrPeriod, 10);   
   double diff = Close[0] - tma;
   double extremeTma = (diff/tmaAtr) / TmaAtrThreshold; 
   
   double n = tmaAtr * 0.1;    
   
   double tmaSlope = ((tma- tmaPrev) / n) ; 
   double tmaSlope1 = ((tmaPrev- tmaPrev2) / n) ; 
   double tmaSlope2 = ((tmaPrev2- tmaPrev3) / n) ;
   double tmaSlopeChange = ((tmaSlope - tmaSlope1) + (tmaSlope1 - tmaSlope2)) / 2.0;
   
      
   paintGeneral("TmaSizeValue", (tmaAtr/Tick) * 4   , NeutralColor);
      
   color c;
   
   if(extremeTma<=-1){c = ExtremeBullColor; } 
   else if(extremeTma>=1){c = ExtremeBearColor; }
   else if(extremeTma>0){c = BearColor; }
   else if(extremeTma<0){c = BullColor; } 
   else {c = NeutralColor; }   
   paintGeneral("ExtremeTMA", extremeTma, c);  
   
   
   if(tmaSlope<-1 * TmaSlopeThreshold){c = ExtremeBearColor; } 
   else if(tmaSlope>TmaSlopeThreshold){c = ExtremeBullColor; } 
   //else {c = NeutralColor; } 
   paintGeneral("TmaSlope", tmaSlope, c,2);  
   
   
   if (ShowSlopeChange)
   {
      if(tmaSlopeChange<0 && priceSlope>0){c = ExtremeBearColor; } 
      else if(tmaSlopeChange<0){c = BearColor; } 
      else if(tmaSlopeChange>0 && priceSlope<0){c = ExtremeBullColor; }
      else if(tmaSlopeChange>0){c = BullColor; } 
      else {c = NeutralColor; } 
      paintGeneral("TmaSlopeChange", tmaSlopeChange* 100, c);
   }
   
   if(ShowLinearPriceChange)
   {
      if(priceSlope<0 && tmaSlopeChange>0){c = ExtremeBullColor; }    
      else if(priceSlope>0 && tmaSlopeChange<0){c = ExtremeBearColor; } 
      else {c = NeutralColor; }    
      paintGeneral("PriceSlope", priceSlope, c);
   }
   
   if(pivotDist>0 && pivotDist < PivotThreshold){c = ExtremeBullColor; } 
   else if (pivotDist>0){c = BullColor; } 
   else if(pivotDist>0 && pivotDist < PivotThreshold){c = ExtremeBullColor; } 
   else if(pivotDist<0){c = ExtremeBearColor; }
   //else if(extremeTma>0){c = BearColor; }
   //else if(extremeTma<0){c = BullColor; } 
   else {c = NeutralColor; }   
   paintGeneral("PivotDistance", pivotDist, c);  
   
   if(ShowHA)
   {
      double ha = (GetHAClose(0) - GetHAOpen(0))/Tick;
      
      if(ha > 0 && extremeTma<=-1){c = ExtremeBullColor; } 
      else if(ha<0 && extremeTma>=1){c = ExtremeBearColor; }
      else if(ha>0){c = BullColor; }
      else if(ha<0 ){c = BearColor; } 
      else {c = NeutralColor; }   
   
      paintGeneral("HeikenAshi", ha, c);   
   }
   
   if(ShowMACrossover)
   {
      double back = getMaBack(Symbol(),0); 
      double front = getMaFront(Symbol(),0);
      double maCrossover = back-front;
      
      if(maCrossover > 0 && extremeTma<=1){c = ExtremeBullColor; } 
      else if(maCrossover<0 && extremeTma<=-1){c = ExtremeBearColor; }
      else if(maCrossover>0){c = BullColor; }
      else if(maCrossover<0 ){c = BearColor; } 
      else {c = NeutralColor; }   
   
      paintGeneral("MaCrossover", maCrossover/Tick, c);
   }
  }
  
  
//Data Retrieval
double getMaBack(string symbol, int timeFrame)
{
   int backIdx = MaShift - MaPeriod;
   //double back = iCustom(symbol,timeFrame,"Ma_Crossover_Lines",MaPeriod,MaShift,0,0);
   double back = iMA( symbol, timeFrame, MaPeriod + backIdx, 0, MODE_LWMA, PRICE_TYPICAL, 0);
   return (back);
}
double getMaFront(string symbol, int timeFrame)
{
   double front = iMA( symbol, timeFrame, MaPeriod, 0, MODE_LWMA, PRICE_TYPICAL, MaShift); 
   return (front);
} 
 
 
double getPriceSlope(string symbol, int timeFrame,int Length)
{

   double SumBars = Length * (Length - 1) * 0.5;
   double SumSqrBars = (Length - 1.0) * Length * (2.0 * Length - 1.0) / 6.0;
   double slope;
   int i=0;
   
   double Sum1 = 0;
   for(i=0;i<=Length-1;i++) Sum1 += i*iMA(NULL,0,1,0,1,PRICE_CLOSE,i);

   double SumY = 0;
   for(i=0;i<=Length-1;i++) SumY += iMA(NULL,0,1,0,1,PRICE_CLOSE,i);

   double Sum2 = SumBars * SumY;
   
   double Num1 = Length * Sum1 - Sum2;
   double Num2 = SumBars * SumBars - Length * SumSqrBars;

   if( Num2 != 0 ) 
	slope = 10000*Num1/Num2;
   else 
	slope = 0; 
    
   if (StringSubstr(Symbol(),3,3) == "JPY") slope = slope / 100;
   return (slope);
}

double getTma(string symbol, int timeFrame, int index)
{
   double dblSum  = (TmaPeriod+1)*iClose(symbol, timeFrame,index);
   double dblSumw = (TmaPeriod+1);
   int jnx, knx;
         
   for ( jnx = 1, knx = TmaPeriod; jnx <= TmaPeriod; jnx++, knx-- )
   {
      dblSum  += ( knx * iClose(symbol, timeFrame,index+jnx) );
      dblSumw += knx;

      if ( jnx <= index )
      {
         dblSum  += ( knx * iClose(symbol, timeFrame,index-jnx) );
         dblSumw += knx;
      }
   }
   
   return( dblSum / dblSumw );
}
 
 
double getNearestPivotDistance()
{
   int index = 0;
   double minDistance = 999999;
   for(int i=0; i < ArraySize(Pivots);i++)
   {
      if(MathAbs(Close[0]-Pivots[i]) < minDistance)
      {
         minDistance = MathAbs(Close[0]-Pivots[i]);
         index = i;
      }      
   } 
   double distance = Close[0]-Pivots[index];
   
   return (distance);
   
}
void GetPivots(string symbol)
{
  int day=TimeDay(Time[0]);
  
  if (day != LastPivotDay)
  {
//---- 
   int shift = iBarShift(symbol,PERIOD_D1,Time[0]);
   
   //i=1;
   //while (TimeDay(Time[_shift+i])==prevDay) i++;
   //i--;
    
   double FH=iHigh(symbol,PERIOD_D1,shift+1);
   double FL=iLow(symbol,PERIOD_D1,shift+1);
   double CL=iClose(symbol,PERIOD_D1,shift+1);
   
   double RANGE = FH-FL;
   
   Pivots[0]=NormalizeDouble((FH+FL+CL)/3.0,Digits); 
   ///else FP=NormalizeDouble((FH+FL+2*CL)/4.0,Digits);
   Pivots[1] = Pivots[0] - (0.382 *  RANGE);
   Pivots[2] = Pivots[0] - (0.618033 *  RANGE);
   Pivots[3] = Pivots[0] - (1 *  RANGE);
   Pivots[4] = Pivots[0] - (1.618033 *  RANGE);
   Pivots[5] = Pivots[0] - (2.618033 *  RANGE);
   Pivots[6] = Pivots[0] + (0.382 *  RANGE);
   Pivots[7] = Pivots[0] + (0.618033 *  RANGE);
   Pivots[8] = Pivots[0] + (1 *  RANGE);
   Pivots[9] = Pivots[0] + (1.618033 *  RANGE);
   Pivots[10] = Pivots[0] + (2.618033 *  RANGE);
   
   LastPivotDay = day;
  }
//----
}
//End Data Retrieval

//Drawing
void paintPrice(double value)
{ 
   int precision = 4;

   if (StringSubstr(Symbol(),3,3) == "JPY") precision = 2;
   ObjectSetText("ExtremeTmaInfo_PriceValue",DoubleToStr(value,precision),Size,Font,NeutralColor);
}    
 
  
 
void paintGeneral(string name, double value, color c, int precision = 1)
{   
   ObjectSetText("ExtremeTmaInfo_" + name,DoubleToStr(value,precision),Size,Font,c);
}    



//----------------------------------------   
void initGraph() 
{
   int bottom = Bottom;
   
   if(ShowHA)
   {
      objectCreate("ExtremeTmaInfo_HeikenAshi",12,bottom,DoubleToStr(9,1),Size,Font,NeutralColor); 
      objectCreate("ExtremeTmaInfo_HeikenAshiLabel",65,bottom,"Heiken Ashi:",Size,Font,NeutralColor); 
      bottom = bottom + (Size * 1.6);
   }
   if(ShowMACrossover)
   {
      objectCreate("ExtremeTmaInfo_MaCrossover",12,bottom,DoubleToStr(9,1),Size,Font,NeutralColor); 
      objectCreate("ExtremeTmaInfo_MaCrossoverLabel",65,bottom,"Ma Crossover:",Size,Font,NeutralColor); 
      bottom = bottom + (Size * 1.6);
   }
   objectCreate("ExtremeTmaInfo_PivotDistance",12,bottom,"9",Size,Font,NeutralColor); 
   objectCreate("ExtremeTmaInfo_PivotDistanceLabel",65,bottom,"Pivot Distance:",Size,Font,NeutralColor); 
   bottom = bottom + (Size * 1.6);  
   
   if (ShowSlopeChange)
   {
      objectCreate("ExtremeTmaInfo_PriceSlope",12,bottom,"9",Size,Font,NeutralColor); 
      objectCreate("ExtremeTmaInfo_PriceSlopeLabel",65,bottom,"Price Change:",Size,Font,NeutralColor); 
      bottom = bottom + (Size * 1.6);   
   }
   if(ShowLinearPriceChange)
   {
      objectCreate("ExtremeTmaInfo_TmaSlopeChange",12,bottom,DoubleToStr(9,1),Size,Font,NeutralColor);
      objectCreate("ExtremeTmaInfo_TmaSlopeChangeLabel",65,bottom,"Slope Change:",Size,Font,NeutralColor);  
      bottom = bottom + (Size * 1.6);   
   }
   objectCreate("ExtremeTmaInfo_TmaSlope",12,bottom,DoubleToStr(9,1),Size,Font,NeutralColor); 
   objectCreate("ExtremeTmaInfo_TmaSlopeLabel",65,bottom,"TMA Slope:",Size,Font,NeutralColor); 
   bottom = bottom + (Size * 1.6);
   objectCreate("ExtremeTmaInfo_ExtremeTMA",12,bottom,DoubleToStr(9,1),Size,Font,NeutralColor); 
   objectCreate("ExtremeTmaInfo_ExtremeTMALabel",65,bottom,"Extreme TMA:",Size,Font,NeutralColor);
   bottom = bottom + (Size * 1.6);
   objectCreate("ExtremeTmaInfo_TmaSizeValue",12,bottom,DoubleToStr(9,1),Size,Font,NeutralColor);
   objectCreate("ExtremeTmaInfo_TmaSizeLabel",65,bottom,"TMA Size:",Size,Font,NeutralColor); 
   bottom = bottom + (Size * 1.6);
   if (ShowExtraInfo)
   { 
      objectCreate("ExtremeTmaInfo_SpreadValue",12,bottom,DoubleToStr(9,1),Size,Font,NeutralColor);
      objectCreate("ExtremeTmaInfo_SpreadValueLabel",65,bottom,"Spread:",Size,Font,NeutralColor);
      bottom = bottom + (Size * 1.6);
      objectCreate("ExtremeTmaInfo_AtrValue",12,bottom,DoubleToStr(9,1),Size,Font,NeutralColor);   
      objectCreate("ExtremeTmaInfo_AtrValueLabel",65,bottom,"Atr(14-day):",Size,Font,NeutralColor);
      bottom = bottom + (Size * 1.6); 
      objectCreate("ExtremeTmaInfo_PriceValue",12,bottom,DoubleToStr(9,1),Size,Font,NeutralColor); 
      objectCreate("ExtremeTmaInfo_PriceValueLabel",65,bottom,"Price:",Size,Font,NeutralColor); 
      bottom = bottom + (Size * 1.6);
   }

   WindowRedraw();
  }
//----------------------------------------   
void deinitGraph() 
  { 
   DeleteObjectsByPrefix("ExtremeTmaInfo_");  
   DeleteObjectsByPrefix("ParadoxInfo_");    
   WindowRedraw();
  }
  
//+------------------------------------------------------------------+
void objectCreate(string name,int x,int y,string text="-",int size=42,
                  string font="Arial",color colour=CLR_NONE)
  {
   ObjectCreate(name,OBJ_LABEL,0,0,0);
   ObjectSet(name,OBJPROP_CORNER,3);
   ObjectSet(name,OBJPROP_COLOR,colour);
   ObjectSet(name,OBJPROP_XDISTANCE,x);
   ObjectSet(name,OBJPROP_YDISTANCE,y);
   ObjectSetText(name,text,size,font,colour);
  } 
  
  
  
  
void DeleteObjectsByPrefix(string Prefix)
  {
   int L = StringLen(Prefix);
   int i = 0; 
   while(i < ObjectsTotal())
     {
       string ObjName = ObjectName(i);
       if(StringSubstr(ObjName, 0, L) != Prefix) 
         { 
           i++; 
           continue;
         }
       ObjectDelete(ObjName);
     }
  }
  //End Drawing
  
  

 
 
 
double GetHAClose(int index)
{
   return((Open[index]+High[index]+Low[index]+Close[index])/4);
}

double GetHAOpen(int index)
{
   //The higher you make this lookback the lower the error with the true Heiken Ashi Open)
   int lookback = 8;
   double open = GetHAClose(index+lookback);
   lookback--;
   for(int j = index + lookback;j > index;j--)
   {
      open = (open + GetHAClose(j)) / 2;
   }
   return (open);
}